@@ -22,6 +22,7 @@
 #include "tilda_terminal.h"
 #include "wizard.h"
 
+
 void copy (gpointer data, guint callback_action, GtkWidget *w);
 void paste (gpointer data, guint callback_action, GtkWidget *w);
 void config_and_update (gpointer data, guint callback_action, GtkWidget *w);
@@ -31,8 +32,8 @@ static GtkItemFactoryEntry menu_items[] = {
     { "/_New Tab", "<Ctrl><Shift>T", add_tab_menu_call,     0, "<Item>",      NULL                  },
     { "/_Close Tab",      NULL,      close_tab,             0, "<Item>",      NULL                  },
     { "/sep1",            NULL,      NULL,                  0, "<Separator>", NULL                  },
-    { "/_Copy",           NULL,      copy,                  0, "<StockItem>", GTK_STOCK_COPY        },
-    { "/_Paste",          NULL,      paste,                 0, "<StockItem>", GTK_STOCK_PASTE       },
+    { "/_Copy",    "<Ctrl><Shift>C",      copy,                  0, "<Item>", NULL        },
+    { "/_Paste",   "<Ctrl><Shift>V",      paste,                 0, "<Item>", NULL       },
     { "/sep1",            NULL,      NULL,                  0, "<Separator>", NULL                  },
     { "/_Preferences...", NULL,      config_and_update,     0, "<StockItem>", GTK_STOCK_PREFERENCES },
     { "/sep1",            NULL,      NULL,                  0, "<Separator>", NULL                  },
@@ -41,7 +42,7 @@ static GtkItemFactoryEntry menu_items[] = {
 
 static gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
 
-void fix_size_settings (tilda_window *tw)
+static void fix_size_settings (tilda_window *tw)
 {
 #ifdef DEBUG
     puts("fix_size_settings");
@@ -69,7 +70,7 @@ void clean_up_no_args ()
     gtk_main_quit ();
 }
 
-void free_and_remove (tilda_window *tw)
+static void free_and_remove (tilda_window *tw)
 {
 #ifdef DEBUG
     puts("free_and_remove");
@@ -94,105 +95,34 @@ void goto_tab (tilda_window *tw, guint i)
     gtk_notebook_set_current_page (GTK_NOTEBOOK (tw->notebook), i);
 }
 
-void goto_tab_1 (tilda_window *tw)
+static gboolean goto_tab_generic (tilda_window *tw, gint tab_number)
 {
 #ifdef DEBUG
-    puts("goto_tab 1");
+    printf ("goto_tab_%d\n", tab_number);
 #endif
 
-    if (g_list_length (tw->terms) > 1)
-        goto_tab (tw, g_list_length (tw->terms)-1);
+    if (g_list_length (tw->terms) > (tab_number-1))
+    {
+        goto_tab (tw, g_list_length (tw->terms) - tab_number);
+        return TRUE;
+    }
+
+    return FALSE;
 }
 
-void goto_tab_2 (tilda_window *tw)
-{
-#ifdef DEBUG
-    puts("goto_tab 2");
-#endif
-
-    if (g_list_length (tw->terms) > 1)
-        goto_tab (tw, g_list_length (tw->terms)-2);
-}
-
-void goto_tab_3 (tilda_window *tw)
-{
-#ifdef DEBUG
-    puts("goto_tab 3");
-#endif
-
-    if (g_list_length (tw->terms) > 2)
-        goto_tab (tw, g_list_length (tw->terms)-3);
-}
-
-void goto_tab_4 (tilda_window *tw)
-{
-#ifdef DEBUG
-    puts("goto_tab 4");
-#endif
-
-    if (g_list_length (tw->terms) > 3)
-        goto_tab (tw, g_list_length (tw->terms)-4);
-}
-
-void goto_tab_5 (tilda_window *tw)
-{
-#ifdef DEBUG
-    puts("goto_tab 5");
-#endif
-
-    if (g_list_length (tw->terms) > 4)
-        goto_tab (tw, g_list_length (tw->terms)-5);
-}
-
-void goto_tab_6 (tilda_window *tw)
-{
-#ifdef DEBUG
-    puts("goto_tab 6");
-#endif
-
-    if (g_list_length (tw->terms) > 5)
-        goto_tab (tw, g_list_length (tw->terms)-6);
-}
-
-void goto_tab_7 (tilda_window *tw)
-{
-#ifdef DEBUG
-    puts("goto_tab 7");
-#endif
-
-    if (g_list_length (tw->terms) > 6)
-        goto_tab (tw, g_list_length (tw->terms)-7);
-}
-
-void goto_tab_8 (tilda_window *tw)
-{
-#ifdef DEBUG
-    puts("goto_tab 8");
-#endif
-
-    if (g_list_length (tw->terms) > 7)
-        goto_tab (tw, g_list_length (tw->terms)-8);
-}
-
-void goto_tab_9 (tilda_window *tw)
-{
-#ifdef DEBUG
-    puts("goto_tab 9");
-#endif
-
-    if (g_list_length (tw->terms) > 8)    
-        goto_tab (tw, g_list_length (tw->terms)-9);
-}
-
-void goto_tab_10 (tilda_window *tw)
-{
-#ifdef DEBUG
-    puts("goto_tab 10");
-#endif
-
-    if (g_list_length (tw->terms) > 9)
-        goto_tab (tw, g_list_length (tw->terms)-10);
-}
+/* These all just call the generic function since they're all basically the same
+ * anyway. Unfortunately, they can't just be macros, since we need to be able to
+ * create a pointer to them for callbacks. */
+gboolean goto_tab_1  (tilda_window *tw) { return goto_tab_generic (tw, 1);  }
+gboolean goto_tab_2  (tilda_window *tw) { return goto_tab_generic (tw, 2);  }
+gboolean goto_tab_3  (tilda_window *tw) { return goto_tab_generic (tw, 3);  }
+gboolean goto_tab_4  (tilda_window *tw) { return goto_tab_generic (tw, 4);  }
+gboolean goto_tab_5  (tilda_window *tw) { return goto_tab_generic (tw, 5);  }
+gboolean goto_tab_6  (tilda_window *tw) { return goto_tab_generic (tw, 6);  }
+gboolean goto_tab_7  (tilda_window *tw) { return goto_tab_generic (tw, 7);  }
+gboolean goto_tab_8  (tilda_window *tw) { return goto_tab_generic (tw, 8);  }
+gboolean goto_tab_9  (tilda_window *tw) { return goto_tab_generic (tw, 9);  }
+gboolean goto_tab_10 (tilda_window *tw) { return goto_tab_generic (tw, 10); }
 
 void next_tab (tilda_window *tw)
 {
@@ -223,21 +153,16 @@ void clean_up (tilda_window *tw)
     exit (0);
 }
 
-void clean_up_no_main (tilda_window *tw)
-{
-#ifdef DEBUG
-    puts("clean_up_no_main");
-#endif
-
-    free_and_remove (tw);
-}
-
 void start_program(tilda_collect *collect)
 {
     int argc;
     char **argv;
 
-    if (g_shell_parse_argv (cfg_getstr (collect->tw->tc, "command"), &argc, &argv, NULL))
+    if (!cfg_getbool (collect->tw->tc, "run_command"))
+    {
+        /* Do nothing here, we'll get taken care of later */
+    }
+    else if (g_shell_parse_argv (cfg_getstr (collect->tw->tc, "command"), &argc, &argv, NULL))
     {
         vte_terminal_fork_command (VTE_TERMINAL(collect->tt->vte_term),
             argv[0], argv, NULL,
@@ -245,15 +170,18 @@ void start_program(tilda_collect *collect)
             TRUE, TRUE, TRUE);
 
         g_strfreev (argv);
+        return; // the early way out
     }
-    else
-    {
-        const gchar *command = g_getenv ("SHELL");
-        vte_terminal_fork_command (VTE_TERMINAL(collect->tt->vte_term),
-            command, NULL, NULL,
-            cfg_getstr (collect->tw->tc, "working_dir"),
-            TRUE, TRUE, TRUE);
-    }
+
+    gchar *command = (gchar *) g_getenv ("SHELL");
+
+    if (command == NULL)
+        command = "/bin/sh";
+
+    vte_terminal_fork_command (VTE_TERMINAL(collect->tt->vte_term),
+        command, NULL, NULL,
+        cfg_getstr (collect->tw->tc, "working_dir"),
+        TRUE, TRUE, TRUE);
 }
 
 void close_tab_on_exit (GtkWidget *widget, gpointer data)
@@ -404,6 +332,41 @@ void status_line_changed (GtkWidget *widget, gpointer data)
     g_print ("Status = `%s'.\n", vte_terminal_get_status_line (VTE_TERMINAL(widget)));
 }
 
+
+void ccopy (tilda_window *tw)
+{
+#ifdef DEBUG
+    puts("ccopy");
+#endif
+    GtkWidget *current_page;
+    GList *list;
+
+    gint pos = gtk_notebook_get_current_page (GTK_NOTEBOOK (tw->notebook));
+
+    current_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (tw->notebook), pos);
+
+    list = gtk_container_get_children ((GtkContainer *) current_page);
+
+    vte_terminal_copy_clipboard ((VteTerminal *) list->data);
+}
+
+void cpaste (tilda_window *tw)
+{
+#ifdef DEBUG
+    puts("cpaste");
+#endif
+    GtkWidget *current_page;
+    GList *list;
+
+    gint pos = gtk_notebook_get_current_page (GTK_NOTEBOOK (tw->notebook));
+
+    current_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (tw->notebook), pos);
+
+    list = gtk_container_get_children ((GtkContainer *) current_page);
+
+    vte_terminal_paste_clipboard ((VteTerminal *) list->data);
+}
+
 void copy (gpointer data, guint callback_action, GtkWidget *w)
 {
 #ifdef DEBUG
@@ -730,3 +693,4 @@ void decrease_font_size (GtkWidget *widget, gpointer data)
 
     adjust_font_size (widget, data, -1);
 }
+
